%------Solving Maximum weight independent set using 
%------Quantum Computer----------------------------
clear all;
%----SETTING UP GRAPH INPUT -----------------------
N = 6; % Number of qubits
%Weight_vector = [15 4 1 4 2 3 2 1 1]; % WEIGHT OF VERTICES
Weight_vector = [15 4 1 4 2 1]; % WEIGHT OF VERTICES
%------------INVOKE ORACLE---------------------------
%---TO COMPUTE ALL POSSIBLE WEIGHT COMBINATIONS------
%---QUANTUM COMPUTER WILL DO IT IN PARALLEL----------
R = 0;
for i = 0:1:power(2,N)-1 % CONSTANT TIME OPERATION FOR QC
    f = fliplr(d2b(i,N));
    R(i+1) = sum(f.*Weight_vector);
end
%-----------FINAL HAMILTONIAN----------------------
Vi = R';
Hf = zeros(power(2,N));
% Hf = Gen_Hamiltonian(Vi);
for i = 1: power(2,N)
    Hf(i,i)=(Vi(i));
end
Hf = -expm(Hf);
% H = hadamard(32);
% Hi = H*Hi*H';
%--------INTERMEDIATE CONSTRAINT ORACLE--------------
Oracle_Vector=0;
for i = 0:1:power(2,N)-1    % CONSTANT TIME OPERATION FOR QC
    f = fliplr(d2b(i,N));
    c1 = f(1)+f(2)<=1;
    c2 = f(1)+f(3)<=1;
    c3 = f(1)+f(4)<=2;
    c4 = f(1)+f(5)<=1;
    c5 = f(2)+f(3)<=1;
    c6 = f(2)+f(4)<=2;
    c7 = f(2)+f(5)<=1;
    c8 = f(3)+f(4)<=1;
    c9 = f(3)+f(5)<=2;
    c10 = f(4)+f(5)<=1;
Oracle_fun_outp = c1 & c2 & c3 & c4 & c5 & c6 & c7 & c8 & c9 & c10;
% Writing oracle
Oracle_Vector(i+1) = power(-1,Oracle_fun_outp+1);
end

%-----------CONSTRAINT HAMILTONIAN----------------------
Vi = Oracle_Vector;
%Hc = Gen_Hamiltonian(-Vi);
Hc = zeros(power(2,N));

for i = 1: power(2,N)
    Hc(i,i)=Vi(i);
end

%Hc = Hadamard(32)*Hc*Hadamard(32);
%-----PREPARE QUANTUM STATE IN INITIAL HAMILTONIAN------
[z1,z2] = find(Hc == 1);
% system_quantum_state = [0;1;zeros(1,62)']; % Ground state of Hc
system_quantum_state = zeros(1,power(2,N));
system_quantum_state(z1) = 1;
system_quantum_state = system_quantum_state';

%--Adiabatic search Hc to Hc + I 
%----Setting parameters---------------------------------
step_size = 1/50;
T = 1;
% counter may used for plotting
c = 0;
% start your engines gentlemen
x = system_quantum_state;
for f = 0 : step_size : T
    c = c+1;
    H = Hc+f*eye(power(2,N));
    x = H*x;
    x = x/norm(x);
    [v,e] = eig(H);
    d = eig(H);
    f11 = v\x;
    f11 = f11/norm(f11);
   % stem(d,abs(f11),'r','*'), xlabel 'payoff', ylabel 'probability of getting this payoff' 
   % F(c) = getframe;
end
%--Adiabatic search Hc + I to Hf
c = 0;
y = x;
x = -x;
%----Setting parameters---------------------------------
% step_size = 1/100;
Hi = H;
ex = system_quantum_state;
for f = 0 : step_size : T
    c = c+1;
    H = (1-f)*Hi + f*Hf;
%     H = (cos((2*pi*f)/(4*T)))*Hi + sin(f*pi/(2*T))*Hf;
    x = H*x;
    sp_gap(c) = spectral_gap(H);
    x = x/norm(x);
    [v,e] = eig(H);
    d = -eig(H);    % so that weight eigen values become+ve
    f11 = v\x;
    f11 = f11;
     %stem(d,abs(f11),'r','*'), xlabel 'Maximum weight', ylabel 'probability of getting this weight' , axis ([-30 20 -0.01 2]),grid on;
         stem(log(d),abs(f11),'r','*'), xlabel 'Maximum weight', ylabel 'probability of getting this weight' , axis ([-10 40 -0.01 2]),grid on;

    F(c) = getframe;
end
    %stem(log(d),abs(f11),'r','*'), xlabel 'Maximum weight', ylabel 'probability of getting this weight' , axis ([-10 40 -0.01 2]),grid on;

[x.*x y R']
plot([2:c],sp_gap(2:length(sp_gap))), xlabel( 'Time units'), ylabel( 'normalized spectral gap'),grid 'on',legend('variation of spectral gap exponentiating eigen values');
